# CML Contributing Guide

Thanks for your interest contributing to CML.
The following are a set of guidelines for contributing.
All suggestions are welcome, including suggestions to the contributing guide.

## Issue Reporting Guidelines

Please submit issues through GitHub Issues.

Issues include:

* Enhancements
* Bugs
* Fixes

## Pull Request Guidelines

Reviewer(s) may ask you to complete additional tasks before your pull request
can be accepted.

## How does compilation work?

1. CML Code
2. AST (Abstract Syntax Tree)
3. IR (Intermediate Representation)
4. Javascript Code

**CML**

We start with a string with CML code.
This is parsed using files in `lib/parser`.
The output of this process is an AST, abstract syntax tree.

**Abstract Syntax Tree**

The AST generated by the parser allows us to create an IR, intermediate representation.

**Intermediate Representation**

The IR is a serializable instruction set for generating the final code.
In this case the IR is formatted as JSON.

**Javascript**

In the final step, we take the IR JSON and generate javascript code that works with our **runtime**.

## Project Structure
* `doc` documentation directory, where we put all .md files for documentation
* `lib` main directory where all code is
    * `build` where we generate an html file from a project
    * `compile` where we generate a javascript file from our IR
    * `parser` where we parse CML into an AST and IR
    * `runtime` client side file that runs compiled javascript
    * `template` template html files for build process
    * `cml.js` main compiler file
    * `cml-browser.js` main compiler file, for browser
* `LICENSE.txt`
* `CODE_OF_CONDUCT.md`
* `README.md`
* `package.json`
